@@ -1,17 +1,17 @@
 import asyncio
 import logging
+from pathlib import Path
 from datetime import datetime
 from typing import Dict, Any, Optional, List, Callable, AsyncGenerator
 
 import uvicorn
 from fastmcp import FastMCP, settings as fastmcp_settings
-from fastmcp.client.client import Client
 from fastmcp.client.transports import (
     SSETransport,
     StreamableHttpTransport,
     ClientTransport,
 )
-from fastmcp.utilities.mcp_config import (
+from fastmcp.mcp_config import (
     MCPConfig,
     RemoteMCPServer,
     StdioMCPServer,
@@ -25,7 +25,7 @@ from sse_starlette.sse import AppStatus
 from codearkt.codeact import CodeActAgent
 from codearkt.llm import ChatMessage
 from codearkt.event_bus import AgentEventBus
-from codearkt.util import get_unique_id, find_free_port
+from codearkt.util import get_unique_id, find_free_port, append_jsonl_atomic
 
 DEFAULT_SERVER_HOST = "0.0.0.0"
 DEFAULT_SERVER_PORT = 5055
@@ -188,8 +188,7 @@ def get_mcp_app(
                 transport = server.to_transport()
 
             assert transport is not None, "Transport is required for the MCP server in the config"
-            client: Client[ClientTransport] = Client(transport=transport)
-            sub_proxy = FastMCP.as_proxy(client)
+            sub_proxy = FastMCP.as_proxy(backend=transport)
             prefix: Optional[str] = None if server_count == 1 else name
             if not add_prefixes:
                 prefix = None
@@ -323,8 +322,10 @@ async def run_batch(
     agent: CodeActAgent,
     mcp_config: Optional[Dict[str, Any]] = None,
     max_concurrency: int = 5,
+    task_timeout: Optional[int] = None,
     additional_tools: Optional[Dict[str, Callable[..., Any]]] = None,
     add_mcp_server_prefixes: bool = True,
+    output_path: Optional[Path] = None,
 ) -> List[str]:
     if not queries:
         return []
@@ -340,12 +341,34 @@ async def run_batch(
 
     async def _run_single(q: str) -> str:
         async with semaphore:
-            return await agent.ainvoke(
-                [ChatMessage(role="user", content=q)],
-                session_id=get_unique_id(),
-                server_host=host,
-                server_port=port,
+            session_id = get_unique_id()
+            task = asyncio.create_task(
+                agent.ainvoke(
+                    [ChatMessage(role="user", content=q)],
+                    session_id=session_id,
+                    server_host=host,
+                    server_port=port,
+                )
             )
+            result: str
+            try:
+                if task_timeout and task_timeout > 0:
+                    result = await asyncio.wait_for(task, timeout=task_timeout)
+                else:
+                    result = await task
+            except asyncio.TimeoutError:
+                task.cancel()
+                try:
+                    await asyncio.wait_for(task, timeout=2)
+                except (asyncio.CancelledError, asyncio.TimeoutError):
+                    pass
+                result = f"Timeout after {task_timeout}"
+            except Exception as e:
+                result = f"Error: {e}"
+            finally:
+                if output_path:
+                    append_jsonl_atomic(output_path, {"query": q, "result": result})
+                return result
 
     try:
         tasks = [_run_single(q) for q in queries]
