@@ -1,8 +1,11 @@
 import asyncio
+import tempfile
+from pathlib import Path
 from textwrap import dedent
 from datetime import datetime
 
 from academia_mcp.tools import arxiv_search
+from academia_mcp.tools import arxiv_download
 
 from codearkt.codeact import CodeActAgent, extract_code_from_text
 from codearkt.llm import ChatMessage, LLM
@@ -282,18 +285,62 @@ class TestCodeActAgent:
             llm=deepseek,
             tool_names=["arxiv_search"],
         )
-        results = await run_batch(
-            ["What is 432412421249 * 4332144219?", "Get the exact title of 2409.06820v4."],
-            agent,
-            {},
-            additional_tools={"arxiv_search": arxiv_search},
-        )
-        assert len(results) == 2, results
+        with tempfile.NamedTemporaryFile(mode="w") as f:
+            results = await run_batch(
+                ["What is 432412421249 * 4332144219?", "Get the exact title of 2409.06820v4."] * 2,
+                agent,
+                additional_tools={"arxiv_search": arxiv_search},
+                max_concurrency=4,
+                output_path=Path(f.name),
+            )
+            text = Path(f.name).read_text()
+            assert text is not None, text
+            assert len(text) > 0, text
+            assert len(text.strip().splitlines()) == 4, text
+        assert len(results) == 4, results
         result1 = str(results[0]).replace(",", "").replace(".", "").replace(" ", "")
         assert "1873272970937648109531" in result1, result1
         result2 = str(results[1]).lower()
         assert "role-playing language models" in result2, result2
 
+    async def test_codeact_multi_agent_batch(self, deepseek: LLM) -> None:
+        agent = CodeActAgent(
+            name="agent",
+            description="Just agent",
+            llm=deepseek,
+            managed_agents=[get_nested_agent()],
+        )
+        queries = ["Get the exact title of 2409.06820v4."] * 5
+        results = await run_batch(
+            queries,
+            agent,
+            max_concurrency=5,
+            task_timeout=600,
+            additional_tools={"arxiv_search": arxiv_search, "arxiv_download": arxiv_download},
+        )
+        assert len(results) == 5, results
+        for result in results:
+            assert "role-playing language models" in str(result).lower(), result
+
+    async def test_codeact_multi_agent_timeout_batch(self, deepseek: LLM) -> None:
+        agent = CodeActAgent(
+            name="agent",
+            description="Just agent",
+            llm=deepseek,
+            managed_agents=[get_nested_agent()],
+        )
+        queries = ["Get the exact title of 2409.06820v4."] * 5
+        results = await run_batch(
+            queries,
+            agent,
+            max_concurrency=2,
+            task_timeout=1,
+            additional_tools={"arxiv_search": arxiv_search, "arxiv_download": arxiv_download},
+        )
+        assert len(results) == 5, results
+        for result in results:
+            assert "timeout" in str(result).lower(), result
+
     async def test_codeact_tool_description_dedent(
         self, deepseek: LLM, mcp_server_test: MCPServerTest
     ) -> None:
@@ -321,3 +368,14 @@ class TestCodeActAgent:
         )
         current_date = datetime.now().strftime("%Y-%m-%d")
         assert current_date in str(result), result
+
+    async def test_codeact_grok_code(self, grok_code: LLM) -> None:
+        agent_name = "agent"
+        agent = CodeActAgent(
+            name=agent_name,
+            description="Just agent",
+            llm=grok_code,
+        )
+        result = await run_query("What is 432412421249 * 4332144219?", agent, {})
+        str_result = str(result).replace(",", "").replace(".", "").replace(" ", "")
+        assert "1873272970937648109531" in str_result, str_result
